!*********************************************************
!MODULE WITH ALL RELEVANT VARIABLES USED IN COMPUTATION
!
!Marina Pavan
!August 4, 2005
!*********************************************************


MODULE mod_computation

use mod_types
use mod_param


implicit none

!**********
!variables:
!**********

!states:
!*******

real(dp), dimension(fulnh,nbeta)::h   !house
real(dp), dimension(fulnh,nbeta)::coll !collateral constraint

integer,dimension(nybz)::index_beta

!real(dp)::min_f
!real(dp)::max_f
!real(dp), dimension(nf)::f  !flat to rent

real(dp)::max_b  !max c. debt
real(dp), dimension(fulnb,nbeta)::b   !collateralized debt

real(dp), dimension(fulnb,nbeta)::ind_b  !indicator function for b>0

integer,dimension(nbeta)::nnb

!aggregate capital:
real(dp),dimension(nk)::K_grid
real(dp),dimension(nhh)::H_grid

!initial capital for simulation:
!real(dp)::iniK
real(dp)::iniKoverL 
!real(dp)::iniH  !initial aggregate housing (declared in mod_param.f90 now)
integer,dimension(2)::indsiniK,indsiniHH,indsiniL,indsiniKoverL 
real(dp),dimension(2)::valsiniK,valsiniHH,valsiniL,valsiniKoverL
integer, dimension(2,nbeta)::indsinib,indsinih
real(dp),dimension(2,nbeta)::valsinib,valsinih

! Matteo May 2007
real(dp)::KoverY
real(dp)::HoverC
real(dp)::CoverY
real(dp)::iniL
real(dp)::iniY
real(dp)::iniC
real(dp)::probsur








!prices:
!********
real(dp),dimension(ns,nk)::R,Wa

real(dp),dimension(ns,nk)::KPRIME,LPRIME  !next period capital and labor

real(dp),dimension(2,ns,nk)::valsKP,valsHP
integer,dimension(2,ns,nk)::indsKP,indsHP


!coefficients to be iterated upon, allocated in state generation subroutine, depending on option_reg value:
real(dp),allocatable::coefK(:)
real(dp),allocatable::coefL(:)


real(dp),dimension(time_length)::totKoverL,totK,totL


real(dp), dimension(ns)::logA  !log of aggregate shock


real(dp),dimension(time_length-TAC)::dipK,dipL   

real(dp), allocatable::exxe(:,:)   !allocated in state_generation subroutine depending on value of option_reg

real(dp), dimension(fulnh,fulnb)::EV !expected value function

!others:
!*******

integer,allocatable::state(:,:)  !index of joint state for zy and beta
integer,allocatable::age_state(:,:)  !age of simulated ind. at each point in time

!counters for distribution computations:
!Marina June 2009
!***************************************
integer,dimension(max_age,time_length)::count_a_people  !at each tm, the number of people of each age
integer, dimension(max_age,nbeta,time_length)::count_ab_people !at each tm, the number of people of a certain age and beta combination
integer, dimension(ns)::count_s  !the total number of periods with a certain aggregate shock, for each aggregate shock
integer, dimension(nbeta)::count_b_people  !the total number of betas at each period (does not change from period to period)

contains

subroutine state_generation

use mod_types
use mod_param


implicit none

integer::ih,ib,iy,icoh,ik,is,tm,j1,age
real(dp)::widths,temp   
real(dp), dimension(nbeta)::mid_h
real(dp)::min_h,minK  !min house
real(dp)::max_h,maxK !max house
real(dp)::max_b

real(dp)::meanbeta


!compute tax to sustain pension:
tax=pension*sum(p_age(ret_age:max_age))/sum(p_age(1:ret_age-1))


!-------------------------------------------
! GENERATE PARAMETERS THAT ARE AGE DEPENDENT
!-------------------------------------------

!generate WEIGHT ON HOUSE as a function of age:
!------------------------------------------------
whouse=whouse_base
iwhouse=1.0_dp-whouse
iiwhouse=1.0_dp+whouse

!generate WEIGHT ON LEISURE as a function of age:
!------------------------------------------------
do age=1,ret_age
tau(age)=tau_base*(coeff_tau0+coeff_tau1*real(age,dp)+coeff_tau2*real(age,dp)**2.0_dp)
itau(age)=1.0_dp+tau(age)
enddo

do age=ret_age+1,max_age
tau(age)=tau_base
itau(age)=1.0_dp+tau(age)
enddo

!generate WEIGHT ON RENTING as a function of age:
!------------------------------------------------
do age=1,max_age
wrent(age)=(wrent_base+coeff_wrent1*real(age,dp))
enddo

!generate MARGINAL UTILITY OF CONSUMPTION that depends on age:
!-------------------------------------------------------------
do age=1,max_age
wbeta(:,age)=(coeff_w0+coeff_w1*real(age,dp)+coeff_w2*real(age,dp)**2.0_dp+coeff_w3*real(age,dp)**3.0_dp+coeff_w4*real(age,dp)**4.0_dp)
enddo

!generate DISCOUNT FACTOR that depends on age:
!---------------------------------------------
do age=1,max_age
betta(1,age)=beta(1)*(coeff1_b0+coeff1_b1*real(age,dp)+coeff1_b2*real(age,dp)**2.0_dp+coeff1_b3*real(age,dp)**3.0_dp)
betta(2,age)=beta(2)*(coeff2_b0+coeff2_b1*real(age,dp)+coeff2_b2*real(age,dp)**2.0_dp+coeff2_b3*real(age,dp)**3.0_dp)
enddo

!------------------------------------------------------------------------------------------------------------


!allocate regression coefficients
allocate(coefK(2*ns),coefL(2*ns))

!allocate regression matrix:
allocate(exxe(time_length-TAC,2*ns))

!********************************
!CAPITAL GRID
!********************************

! Matteo May 2007
! To construct grid for K, need first a plausible range of values for L
! Matteo2 multiply iniL by lbar

probsur=0.98_dp;
KoverY=(alfa/(1.0_dp/(beta(nbeta)*probsur)-ideltaK))
HoverC=(whouse_base/(1.0_dp-(beta(nbeta)*probsur)*idelta))
CoverY=(1.0_dp-deltaK*KoverY)/(1.0_dp+delta*HoverC)

! I put a correction factor here for ss hours
iniL=lbar*(ialfa/(ialfa+tau_base*CoverY))

!iniK=iniL*(KoverY**iialfa)
iniY=(iniK**alfa)*(iniL**ialfa)
iniC=CoverY*iniY

iniKoverL=(KoverY**iialfa)


print*, "------------"
print*, "NUMBERS BELOW ONLY USED FOR INITIAL CONDITIONS IN SIMUL"
print*, "KoverY=",KoverY
print*, "HoverC=",HoverC
print*, "CoverY=",CoverY
print*, "iniL=",iniL
print*, "(calculated in mod_param) iniK=",iniK
print*, "iniY=",iniY
print*, "iniC=",iniC
print*, "*iniH=",iniH
print*, "iniKoverL=",iniKoverL
print*, "------------"


!print*, "NUMBER BELOW NEEDED FOR GUESS OF PRICE FUNCTION"
!print*, "iniKL (set by hand in mod_param if =/= from above)=",iniKL


!xxx KoverLgrid change
!now here there is no KoverLgrid generation anymore - it is in the subroutine below


!if nbeta=1
!ibeta(1)=1.0_dp-beta(1)
!iibeta(1)=1.0_dp/ibeta(1)

!*******************************
!HOUSE - function of whouse(2)
!*******************************

!if (nbeta>1) then
!meanbeta=ppbeta(1)*beta(1)+ppbeta(2)*beta(2)
!else
!meanbeta=mbeta
!endif

h=0.0_dp


!if (nz>1) then
!temp=whouse(2)/(iwhouse(2)*(1.0_dp-mbeta(2)*idelta))
!temp=((1.0_dp-(alfa*meanbeta*deltaK/(1.0_dp-meanbeta*ideltaK)))/(delta+(1.0_dp-meanbeta*idelta)/whouse(2)))*(alfa*meanbeta/(1.0_dp-meanbeta*ideltaK))**(alfa/ialfa)
!else
!temp=whouse(1)/(iwhouse(1)*(1.0_dp-mbeta(1)*idelta))
!temp=((1.0_dp-(alfa*meanbeta*deltaK/(1.0_dp-meanbeta*ideltaK)))/(delta+(1.0_dp-meanbeta*idelta)/whouse))*(alfa*meanbeta/(1.0_dp-meanbeta*ideltaK))**(alfa/ialfa)
!endif

!temp=(iniK**alfa-deltaK*iniK)/(delta+(1.0_dp-beta(2)*idelta)/whouse)

! to assign to each beta, Marina23/01/08:
mid_h(1)=iniH
mid_h(2)=iniH


!iniH=mid_h  
!print*, "iniH=",iniH

do j1=1,nbeta

if (nh(j1)>1) then


max_h=(1.0_dp+span_hp(j1))*mid_h(j1)
min_h=(1.0_dp-span_hn(j1))*mid_h(j1)

widths=(max_h-min_h)/real((nh(j1)-2),dp)


do ih=1,nh(j1)

    if (ih==1) then
        h(ih,j1)=minhome
        coll(ih,j1)=mcoll*h(ih,j1)
    else
        h(ih,j1)=min_h+(real((ih-2),dp)*widths)**curvh
        coll(ih,j1)=mcoll*h(ih,j1)
    endif
    
enddo

else  !only 1 point: no choice for housing

h(1,:)=0.0_dp
mid_h(:)=0.5_dp
max_h=0.0_dp

endif

enddo  !end beta

!construct aggregate H grid:

max_h=(1.02_dp+span_hh)*mid_h(2)
min_h=(1.02_dp-span_hh)*mid_h(2)

widths=(max_h-min_h)/real((nhh-1),dp)


do ih=1,nhh

H_grid(ih)=min_h+(real((ih-1),dp)*widths)**curvh

enddo


if (nhh>1) then
call basefun(H_grid,nhh,iniH,valsiniHH(:),indsiniHH(:))
else
indsiniHH(:)=1
valsiniHH(:)=0.5_dp
endif

do j1=1,nbeta
call basefun(h(1:nh(j1),j1),nh(j1),iniH,valsinih(:,j1),indsinih(:,j1))
enddo

!************
!COLL. DEBT
!************

do j1=1,nbeta

if (nb(j1)>1) then

if (nh(j1)>1) then

max_b=max_s(j1)

if (mcoll>0.0_dp) then
!note: max_h here is the max of aggregate H:
nnb(j1)=dint(real((nb(j1)*max_s(j1)/(4.0_dp*max_h+max_s(j1))),dp))-1
else
nnb(j1)=nb(j1)-1
endif

else

nnb(j1)=nb(j1)-1
b(nb,j1)=0.0_dp

endif

ind_b(:,j1)=0.0_dp  !saving/borrowing indicator

do ib=1,nnb(j1)

b(nnb(j1)-ib+1,j1)=max_s(j1)*(real((ib),dp)/real((nnb(j1)),dp))**curvs

enddo

b(:,j1)=-b(:,j1)

if (nh(j1)>1) then

max_b=mcoll*h(nh(j1),j1)

if (mcoll>0.0_dp) then
do ib=nnb(j1)+1,nb(j1)

b(ib,j1)=max_b*(real((ib-1-nnb(j1)),dp)/real((nb(j1)-nnb(j1)-1),dp))**curvb

ind_b(ib,j1)=1.0_dp  !index for positive b (debt)

enddo
else
b(nb,j1)=max_b
endif

endif


else  !no choice for b

b(1,:)=0.0_dp

endif

nnb(j1)=nnb(j1)+1


call basefun(b(1:nb(j1),j1),nb(j1),-iniK,valsinib(:,j1),indsinib(:,j1))

enddo   !end beta


!************
!UNCOLL. DEBT
!************

!if (nd>1) then

!	do id=1,nd

!	d(id)=max_d*(real((id-1),dp)/real((nd-1),dp))**curvd

!	enddo 


!else  !no choice for d

!d(1)=0.0_dp

!endif


!*******************************
! Store parameter settings
!*******************************

open (unit = 1312, file = 'h.txt', status = 'replace')
do j1=1,nbeta
write (1312,  '(f16.4)') (h(1:nh(j1),j1))
enddo
close (unit=1312)

open (unit = 1312, file = 'b.txt', status = 'replace')
do j1=1,nbeta
write (1312,  '(f16.4)') (b(1:nb(j1),j1))
enddo
close (unit=1312)

open (unit = 1312, file = 'K_grid.txt', status = 'replace')
write (1312,  '(f16.4)') (K_grid)
close (unit=1312)

open (unit = 1312, file = 'H_grid.txt', status = 'replace')
write (1312,  '(f16.4)') (H_grid)
close (unit=1312)

open (unit = 1312, file = 'beta.txt', status = 'replace')
write (1312,  '(f16.4)') (beta)
close (unit=1312)

open (unit = 1312, file = 'betta.txt', status = 'replace')
do ib=1,nbeta
write (1312,  '(f16.4)') (betta(ib,:))
enddo
close (unit=1312)

open (unit = 1312, file = 'wbeta.txt', status = 'replace')
do ib=1,nbeta
write (1312,  '(f16.4)') (wbeta(ib,:))
enddo
close (unit=1312)

open (unit = 1312, file = 'tau.txt', status = 'replace')
write (1312,  '(f16.4)') (tau)
close (unit=1312)

open (unit = 1312, file = 'lbar.txt', status = 'replace')
write (1312,  '(f16.4)') (lbar)
close (unit=1312)

open (unit = 1312, file = 'sigma.txt', status = 'replace')
write (1312,  '(f16.4)') (sigma)
close (unit=1312)

open (unit = 1312, file = 'JEI.txt', status = 'replace')
write (1312,  '(f16.4)') (whouse)
close (unit=1312)

open (unit = 1312, file = 'wrent.txt', status = 'replace')
write (1312,  '(f16.4)') (wrent)
close (unit=1312)

open (unit = 1312, file = 'sigma_z.txt', status = 'replace')
write (1312,  '(f16.4)') (stdevy)
close (unit=1312)

open (unit = 1312, file = 'rho_z.txt', status = 'replace')
write (1312,  '(f16.4)') (y_rho)
close (unit=1312)

open (unit = 1312, file = 'mean_y.txt', status = 'replace')
write (1312,  '(f16.4)') (mean_y)
close (unit=1312)

open (unit = 1312, file = 'M.txt', status = 'replace')
write (1312,  '(f16.4)') (mcoll)
close (unit=1312)

open (unit = 1312, file = 'minco.txt', status = 'replace')
write (1312,  '(f16.4)') (minco)
close (unit=1312)

open (unit = 1312, file = 'MAX_S.txt', status = 'replace')
write (1312,  '(f16.4)') (max_s)
close (unit=1312)

open (unit = 1312, file = 'DELTAH.txt', status = 'replace')
write (1312,  '(f16.4)') (delta)
close (unit=1312)

open (unit = 1312, file = 'DELTAK.txt', status = 'replace')
write (1312,  '(f16.4)') (deltaK)
close (unit=1312)
open (unit = 1312, file = 'ALPHA.txt', status = 'replace')
write (1312,  '(f16.4)') (alfa)
close (unit=1312)

open (unit = 1312, file = 'rho_a.txt', status = 'replace')
write (1312,  '(f16.4)') (rhoa)
close (unit=1312)
open (unit = 1312, file = 'sigma_a.txt', status = 'replace')
write (1312,  '(f16.4)') (stdevA)
close (unit=1312)

open (unit = 1312, file = 'NPUP.txt', status = 'replace')
write (1312, '(I6)') (num_pop_pup)
close (unit=1312)

open (unit = 1312, file = 'T.txt', status = 'replace')
write (1312,  '(I5)') (time_length)
close (unit=1312)

open (unit = 1312, file = 'ret_age.txt', status = 'replace')
write (1312,  '(I5)') (ret_age)
close (unit=1312)

open (unit = 1312, file = 'T1.txt', status = 'replace')
write (1312,  '(I5)') (time_length-time_start)
close (unit=1312)

open (unit = 1312, file = 'FRACH.txt', status = 'replace')
write (1312,  '(f16.4)') (frach)
close (unit=1312)
open (unit = 1312, file = 'COSTH.txt', status = 'replace')
write (1312,  '(f16.4)') (costh)
close (unit=1312)

open (unit = 1312, file = 'pension.txt', status = 'replace')
write (1312,  '(f16.4)') (pension)
close (unit=1312)

open (unit = 1312, file = 'tax.txt', status = 'replace')
write (1312,  '(f16.4)') (tax)
close (unit=1312)

open (unit = 1312, file = 'wbequest.txt', status = 'replace')
write (1312,  '(f16.4)') (wbequest)
close (unit=1312)

open (unit = 1312, file = 'rpremium.txt', status = 'replace')
write (1312,  '(f16.4)') (rpremium)
close (unit=1312)



!*****************************
!generate also random earnings
!realizations for simulation
!*****************************

!if (pbeta(1,1)<1.0_dp) then
call random_gen1
!else
!call random_gen
!endif

open (unit = 9141, file = 'sim_beta.txt', status="replace")
do tm=time_start+1,time_length
do j1=idnint((num_pop-num_pop1)*ppbeta(1)),idnint((num_pop-num_pop1)*ppbeta(1))+num_pop1-1
write (9141,  '(f16.4)') (beta(index_beta(state(j1,tm))))
enddo
enddo
close (unit=9141)

end subroutine state_generation

! Kgrid change: new subroutine to generate K grid given minK and maxK
subroutine Kgrid_generation(minK,maxK)

use mod_types
use mod_param

implicit none

real(dp),intent(in)::minK,maxK
real(dp)::widths
integer::ik


print*, "minK=", minK
print*, "maxK=", maxK

widths=(maxK-minK)/real(nk-1,dp)

K_grid=(/(ik*widths,ik=0,nk-1)/)
K_grid=K_grid+minK


!for first period in simulations:
call basefun(K_grid,nk,iniK,valsiniK(:),indsiniK(:))

open (unit = 1312, file = 'K_grid.txt', status = 'replace')
write (1312,  '(f16.4)') (K_grid)
close (unit=1312)


end subroutine Kgrid_generation

!***************************************************************************************
!GRID_X IS A GRID WITH NPX POINTS, X IS A NUMBER IN THE INTERVAL DEFINED BY THE GRID.
!THIS SUBROUTINE RETURNS THE TWO GRID POINTS THAT ARE THE CLOSEST TO X
!(THE ONE IMMEDIATELY BEFORE AND THE ONE IMMEDIATELY AFTER X): THE INDEXES ARE IN INDS,
!the distance b/w them and x are in vals(1) and vals(2)
!*************************************************************************************** 
subroutine basefun (grid_x,npx,x,vals,inds) 
implicit none
	
! this subroutine returns the values and the indices of the two basis
! functions that are positive on a given x in the grid_x

real(dp),intent(in) :: x
integer , intent(in):: npx
real(dp), intent(in) :: grid_x (npx)
real(dp), intent(out) ::vals(2)
integer ,intent(out) ::inds(2)
integer :: i,ju,jl,jm

jl=1      
ju=npx   

do
	if (ju-jl<=1) exit
		jm=(ju+jl)/2

	if (x>=grid_x(jm)) then
		jl=jm
	else
		ju=jm
	endif

end do

	i=jl+1

	vals(2)=( x-grid_x(i-1) )/(grid_x(i)-grid_x(i-1))
	vals(1)=( grid_x(i)-x )/(grid_x(i)-grid_x(i-1))
	inds(2)=i
	inds(1)=i-1
	
	if (vals(2)<0.0_dp) then
	vals(2)=0.0_dp
	vals(1)=1.0_dp
	endif
	if (vals(1)<0.0_dp) then
	vals(2)=1.0_dp
	vals(1)=0.0_dp
	endif
	

end subroutine basefun

SUBROUTINE polint(xa,ya,x,y,dy)
use mod_types
use mod_nrutil, only: assert_eq,iminloc,nrerror
implicit none
real(dp),dimension(:),intent(in)::xa,ya
real(dp),intent(in)::x
real(dp), intent(out)::y,dy
!given arrays xa and ya of dimension n, and given a value x, this routine (Numerical Recipes) returns a value y,
!and an error estimate dy. if p(x) is the polynomial of degree n-1 such that p(xa(j))=ya(j), j=1,..,n, then the
!returned value y=p(x)
integer::m,n,nns
real(dp),dimension(size(xa))::c,d,den,ho

!n=assert_eq(size(xa),size(ya),'polint')
n=size(xa)
c=ya      !initialize the tableau of c's and d's
d=ya
ho=xa-x
nns=iminloc(dabs(x-xa))    !find index nns of closest table entry
y=ya(nns)     !initial approximation to y
nns=nns-1
do m=1,n-1   !for each column of the tableau
    den(1:n-m)=ho(1:n-m)-ho(1+m:n)    !we loop over the current c's and d's and update them
    if ( any(den(1:n-m)==0.0_dp)) &
        call nrerror('polint: calculation failure')
        !This error can occur only if two input xa's are (to whithin roundoff) identical
        den(1:n-m)=(c(2:n-m+1)-d(1:n-m))/den(1:n-m)
        d(1:n-m)=ho(1+m:n)*den(1:n-m)  !c's and d's are updated
        c(1:n-m)=ho(1:n-m)*den(1:n-m)
        if (2*nns<n-m) then
        dy=c(nns+1)
        else
        dy=d(nns)
        nns=nns-1
        endif
        y=y+dy
enddo
end subroutine polint

subroutine polin4(x1a,x2a,x3a,x4a,ya,x1,x2,x3,x4,y,dy)
use mod_types
use mod_nrutil, only: assert_eq
implicit none
real(dp),dimension(:),intent(in)::x1a,x2a,x3a,x4a
real(dp),dimension(:,:,:,:),intent(in)::ya
real(dp),intent(in)::x1,x2,x3,x4
real(dp),intent(out)::y,dy
!given arrays x1a,..,x4a of different lengths, and the matrix ya tabulated at the grid points of x1a,..,x4a,
!and given values x1,..,x4 of the independent variables, this routine returns an interpolated function value y
integer::j1,j2,m1,m2,m3,m4
real(dp),dimension(size(x1a),size(x2a))::ytemp
real(dp),dimension(size(x3a),size(x4a))::ytemp1

!m1=assert_eq(size(x1a),size(ya,1),'polin4: 1 dimension')
!m2=assert_eq(size(x2a),size(ya,2),'polin4: 2 dimension')
!m3=assert_eq(size(x3a),size(ya,3),'polin4: 3 dimension')
!m4=assert_eq(size(x4a),size(ya,4),'polin4: 4 dimension')

m1=size(x1a)
m2=size(x2a)
m3=size(x3a)
m4=size(x4a)

do j1=1,m1
    do j2=1,m2
       ytemp1=ya(j1,j2,:,:)
       call polin2(x3a,x4a,ytemp1,x3,x4,ytemp(j1,j2),dy)
    enddo  !end j2
enddo
call polin2(x1a,x2a,ytemp,x1,x2,y,dy)

end subroutine polin4
            
subroutine polin2(x1a,x2a,ya,x1,x2,y,dy)
use mod_types
use mod_nrutil, only: assert_eq
implicit none
real(dp),dimension(:),intent(in)::x1a,x2a
real(dp),dimension(:,:),intent(in)::ya
real(dp),intent(in)::x1,x2
real(dp),intent(out)::y,dy
!given arrays x1a,. of different lengths, and the matrix ya tabulated at the grid points of x1a,.
!and given values x1,. of the independent variables, this routine returns an interpolated function value y
integer::j1,m1,m2
real(dp),dimension(size(x1a))::ytemp
real(dp),dimension(size(x2a))::ytemp1

!m1=assert_eq(size(x1a),size(ya,1),'polin2: 1 dimension')
!m2=assert_eq(size(x2a),size(ya,2),'polin2: 1 dimension')
m1=size(x1a)
m2=size(x2a)

do j1=1,m1
   ytemp1=ya(j1,:)
   call polint(x2a,ytemp1,x2,ytemp(j1),dy)
enddo
call polint(x1a,ytemp,x1,y,dy)

end subroutine polin2
            

    

    



end module mod_computation
