!*********************************************************
!MODULE WITH ALL RELEVANT PARAMETERS USED IN COMPUTATION
!
!Marina Pavan and Matteo Iacoviello
!November 26, 2007
!*********************************************************


MODULE mod_param

use mod_types

implicit none


!**********
! VARIABLES
!**********

!number of periods in life cycle:
integer, parameter::max_age=70  ! total number of decision periods 21-80
integer, parameter::ret_age=46  ! first period into retirement 66-80
integer, parameter::pery=1  !number of years per period

!age distribution
real(dp), dimension(max_age)::p_age   !age distribution
real(dp), dimension(max_age)::p_surv  !survival probability b/w age a and age a+1

!assigned parameters values:
real(dp), parameter::mbeta=0.97_dp 
real(dp), parameter::span_beta=0.029_dp
integer, parameter::nbeta=2
real(dp), dimension(nbeta),parameter::beta=(/mbeta-span_beta,mbeta+span_beta/) 

!discount factor
real(dp), dimension(nbeta),parameter::ibeta=1.0_dp-beta
real(dp), dimension(nbeta),parameter::iibeta=(/1.0_dp/ibeta(1),1.0_dp/ibeta(2)/)
!calibrated distribution betas
real(dp), dimension(nbeta),parameter::ppbeta=(/0.66_dp,0.34_dp/) 
real(dp),parameter::pi_alfa=0.0_dp   
!transition probability for beta 
real(dp),dimension(nbeta,nbeta),parameter:: pbeta=reshape((/1.0_dp-pi_alfa,pi_alfa*ppbeta(1)/ppbeta(2),pi_alfa,1.0_dp-pi_alfa*ppbeta(1)/ppbeta(2)/),(/nbeta,nbeta/))  




!****************************************
!AGE DEPENDENT COEFFICIENTS (changed in mod_computation)
!**************************************** 

! Coefficients on BETA
real(dp),dimension(nbeta,max_age)::betta  !discount factor that depends on age, constructed in mod_computation
real(dp), parameter::coeff1_b0= 1.00_dp  !constant
real(dp), parameter::coeff1_b1= 0.00_dp  !coeff on age
real(dp), parameter::coeff1_b2= 0.00_dp  !coeff on age^2
real(dp), parameter::coeff1_b3= 0.00_dp  !coeff on age^3
real(dp), parameter::coeff2_b0= 1.00_dp  !constant
real(dp), parameter::coeff2_b1= 0.00_dp  !coeff on age
real(dp), parameter::coeff2_b2= 0.00_dp  !coeff on age^2
real(dp), parameter::coeff2_b3= 0.00_dp  !coeff on age^3

! Coefficients of MARGINAL UTILITY OF CONSUMPTION
! NOHS: y = -0.000000566x4 + 0.000124046x3 - 0.009042398x2 + 0.222516706x + 0.776992397
real(dp),dimension(nbeta,max_age)::wbeta  !discount factor that depends on age, constructed in mod_computation
real(dp), parameter::coeff_w0=  0.776992397_dp  !constant
real(dp), parameter::coeff_w1=  0.222516706_dp  !coeff on age
real(dp), parameter::coeff_w2= -0.009042398_dp  !coeff on age^2
real(dp), parameter::coeff_w3=  0.000124046_dp  !coeff on age^3
real(dp), parameter::coeff_w4= -0.000000566_dp  !coeff on age^4

! Weight on RENT and coefficient on rent 
real(dp), dimension(max_age)::wrent
real(dp), parameter::wrent_base=   0.8381_dp  !weight on housing in utility function for tenant
real(dp), parameter::coeff_wrent1= 0.00_dp  !constant

! Time endowment and weight on disutility of hours TAU
! y = 0.00016x2 - 0.01007x + 1.07253
real(dp), parameter::lbar=2.65_dp !time endowment Matteo2
real(dp), parameter::tau_base=1.65_dp  !disutility of hours worked  
real(dp), dimension(max_age)::tau,itau
real(dp), parameter::coeff_tau0=  1.00_dp  !constant
real(dp), parameter::coeff_tau1=  0.00_dp  !coeff on age
real(dp), parameter::coeff_tau2=  0.00_dp  !coeff on age^2



!****************************************
! SHOCKS
!**************************************** 

! AGGREGATE shock:
real(dp), parameter::stdevA=0.01476_dp
real(dp),parameter::rhoA=0.925_dp    
integer, parameter::ns=7 !number of aggregate shocks
real(dp),dimension(ns)::Ags
real(dp), parameter::width_A=2.1_dp  
!rule of thumb for width_A: (ns,width_A): (2,0.8),(3,1.4),(5,1.85),(7,2.1)
real(dp),dimension(ns,ns)::pAA
integer, parameter::iniA=1  !initial aggregate shock in computation


! IDIOSYNCRATIC shock
real(dp),parameter::stdevy=0.30_dp  !standard deviation of income
real(dp),parameter::y_rho=0.9_dp   !persistence parameter
real(dp),parameter::mean_y=1.0_dp  !mean y
integer, parameter::nny=3  !number of y idiosyncratic shock
real(dp), parameter::width_y=1.4_dp  !ymin=-(width_y)*var(y), ymax=width_y*var(y)
!rule of thumb for width_y: (ny,width_y): (2,0.8),(3,1.4),(5,1.85),(7,2.1)
integer, parameter::ny=nny*ns !number of combination of idiosyncratic and aggregate shocks


! AGE DEPENDENT PRODUCTIVITY:
! HANSEN 21-65
!real(dp),parameter::coeff0=1.0_dp,coeff1=0.0_dp,coeff2=0.0_dp
real(dp),parameter::coeff0=0.732586_dp, coeff1=0.044929_dp,coeff2=-0.000741_dp 

! Retirement earnings:
real(dp),parameter::pension=0.40_dp


!********************************
! OTHER STRUCTURAL PARAMETERS
!********************************

! BORROWING
real(dp), parameter::mcoll=0.75_dp   ! collateral requirement
real(dp), parameter::minco=0.25_dp   ! collateral on income
real(dp), parameter::rpremium=0.00_dp !interest premium to pay on borrowing

! Bequests, housing weights, risk aversion
real(dp), dimension(nbeta),parameter::sigma=(/1.0_dp,1.0_dp/), isigma=1.0_dp-sigma  !risk aversion
real(dp), parameter::wbequest=0.0_dp  !weight on bequest
real(dp), parameter::sharedbequest_dummy=0.0_dp   ! =0 if bequests left for family

real(dp), parameter::whouse_base=0.15_dp
real(dp), dimension(max_age)::whouse,iwhouse,iiwhouse  !share on the house  in log, 1-whouse, 1+whouse

real(dp), parameter::delta=0.05_dp,idelta=1.0_dp-delta  !depreciation rate for house
real(dp), parameter::costh=0.05_dp  !transaction cost as % of h
real(dp), parameter::frach=0.00_dp  !threshold of change h that entails transaction cost

! Production sector:
real(dp),parameter::alfa=0.26_dp,ialfa=1.0_dp-alfa,iialfa=1.0_dp/ialfa
real(dp),parameter::deltak=0.0905_dp,ideltak=1.0_dp-deltak


!****************************
! COMPUTATION
!****************************
! Grid for aggregate KL
integer, parameter::soe_dummy=0            ! =1 IF small open economy soe
real(dp), parameter::interest_soe=1.033_dp ! small open economy interest rate (valid with soe_dummy=1)

integer, parameter::nk=11                  ! dimension K_grid

! Grid small h
real(dp), parameter::minhome=0.0_dp    !minimum home size allowed for 
real(dp), parameter::iniH=2.2_dp    ! central value of H for calculation of grid
real(dp), dimension(nbeta), parameter::span_hp=(/1.50_dp,1.50_dp/)  ! max h = (1+span_hp)*mean_h
real(dp), dimension(nbeta), parameter::span_hn=(/0.00_dp,0.00_dp/)  ! min h = (1-span_hn)*mean_h
integer, dimension(nbeta),parameter::nh=(/25,25/)          !number of points in grid for house
integer, parameter::fulnh=25  !TO PUT IN HERE IS THE MAX OF THE NH
real(dp), parameter::curvh=1.0_dp   !curvature for h

! Grid b
real(dp), dimension(nbeta),parameter::min_b=(/ 0.0_dp,  0.0_dp/)   !min col. debt (zero)
real(dp), dimension(nbeta),parameter::max_s=(/10.0_dp,100.0_dp/)  !maximum savings
integer, dimension(nbeta),parameter::nb=(/160,400/)       !grid for collateralized debt 
integer, parameter::fulnb=400       !TO PUT IN HERE THE MAX OF THE NB
real(dp), parameter::curvb=1.0_dp   !"curvature" to make b grid more sparse at the end
real(dp), parameter::curvs=1.6_dp   !curvature to make saving (negative d) grid more sparse at the end

! Others:
real(dp),parameter::cmin=(1E-300_dp)             !min consumption
real(dp),parameter::small=(1E-3_dp)             !Initial convergence crit for value function iteration
real(dp),parameter::smallino=(0.01_dp)          !convergence crit for market clearing iterations
real(dp),parameter::smalletto=(1E-6_dp)         !convergence crit for stat distr iter
real(dp),parameter::smallissimo=-10000000.0_dp  !- infinity for utility function

integer::num_pop
integer, parameter::num_pop_pup=90000           !for simulation (solution)
integer, parameter::Time_length=5000            !for simulation
integer,parameter::num_pop1=300                 !for simulation (final outputting), assigned together with num_pop in random_gen1
integer, parameter::Time_start=Time_length-100  ! number of periods to keep 
integer, allocatable::indbetalow(:,:),indbetahigh(:,:)  ! indicator =1 if beta is low, =0 otherwise
integer, parameter::TAC=500
integer, parameter::guy1=1                      !guys to save in simulation
integer::guy2

integer, parameter::max_iter=8   !max iterations for "iterations" on gammas



!**********************************
!REGRESSIONS
!**********************************

! FOR SMALL_OPEN_ECONOMY
!real(dp),dimension(2*ns)::coefK_ini=(/iniKL, iniKL, iniKL, iniKL, iniKL, 0.0_dp, 0.0_dp, 0.0_dp, 0.0_dp, 0.0_dp/)

 
! Range for K
real(dp), parameter::span_k=0.22_dp
real(dp), parameter::iniK=3.3_dp


!For use with mod_param.f90 (ns=7)
real(dp),dimension(2*ns)::coefK_ini=(/ &
0.6389_dp, 0.6705_dp, 0.6709_dp, 0.6872_dp, 0.6843_dp, 0.6653_dp, 0.6474_dp, &
0.7923_dp, 0.7860_dp, 0.7895_dp, 0.7878_dp, 0.7921_dp, 0.8012_dp, 0.8102_dp /)
 
real(dp),dimension(2*ns)::coefL_ini=(/ &
1.3240_dp, 1.2990_dp, 1.2654_dp, 1.2675_dp, 1.2962_dp, 1.2978_dp, 1.3144_dp, &
-0.0670_dp, -0.0578_dp, -0.0461_dp, -0.0458_dp, -0.0535_dp, -0.0529_dp, -0.0567_dp /)

!**********************************
! PARAMETERS THAT DO NOT CHANGE FOR NOW
!**********************************
real(dp)::tax

integer, parameter::nhh=1 !dimension H_grid
real(dp), parameter::span_hh=0.075_dp

!fixed effect in income
integer, parameter::nz=1
real(dp), parameter::mean_z=1.0_dp  !mean fixed effect
real(dp), parameter::stdevz=0.2_dp  !stdev fixed effect
real(dp), parameter::width_z=1.0_dp  !zmax=width_z*(stdevz), zmin=-zmax
integer, parameter::nybz=ny*nbeta*nz

!KoverLgrid change
real(dp), parameter::wKold=0.5_dp  !weight on old grid minK and MaxK when forming new grid at each iteration (in iterations)

! UNNEEDED PARAMETERS
integer, parameter::maxiter=60   !max number of iterations in value function iteration
integer, parameter::num_iter=20  !number of iterations in Howard improvement algorithm

!*********************************************************************
!DUMMY TO CHANGE DEPENDING ON WHETHER WE RUN ON UNIX OR ON COMPAQ
!*********************************************************************
INTEGER, PARAMETER::UNIXD=1  !=0 IF RUNNING ON COMPAQ, =1 IF RUNNING ON UNIX




END module mod_param

