!*********************************************************
!subroutine that generates random numbers
!from uniform distribution
!and assigns state (index of joint y and betas)
!+ assigns a state for age according to age distribution
!*********************************************************


subroutine random_gen1


USE mod_types
USE mod_param
USE mod_computation
USE mod_utils
!use NUMERICAL_LIBRARIES


implicit none

!variables:
!**********

integer,dimension(1)::iseed  !seed
integer::j1,j2,j3,tm,it,iy,iz,m,ij,is,age,ib
integer, dimension(nny)::county,counti,excess
real(dp),allocatable::rn(:)
real(dp),dimension(time_length)::rA
real(dp),dimension(nny)::pio
real(dp),dimension(nny,nny)::pia
real(dp),dimension(ns,ns)::piaa
integer::indA  !index of aggregate state
integer,allocatable::statey(:,:)  !index of idios. state

integer, dimension(max_age)::pointert !"pointers" to assign age state in period 1
integer::check_dead,check_alive,countage,excess_dead
integer,dimension(max_age)::check_age1
real(dp)::numero

integer,allocatable::integer_numbers(:),ind(:)
real(dp),allocatable::rnn(:),integer_numbers1(:)

integer::ier,attention
integer, parameter::kflag=2  !returns indexes of sorted vector in ind, without changing cohtemp


!******************
!assign AGE STATE 
!******************

!first period age
!*****************
pointert(1)=idnint(p_age(1)*num_pop_pup)

do age=2,max_age
pointert(age)=pointert(age-1)+idnint(p_age(age)*num_pop_pup)
enddo

!***************************************************************
!ASSIGN NUM_POP so that law of large numbers is satisfied!!!
!***************************************************************

num_pop=pointert(max_age)

allocate(state(num_pop,time_length),age_state(num_pop,time_length))

print*, "num_pop=",num_pop

open (unit = 1312, file = 'N.txt', status = 'replace')
write (1312, '(I6)') (num_pop)
close (unit=1312)

open (unit = 1312, file = 'N1.txt', status = 'replace')
write (1312, '(I5)') (num_pop1)
close (unit=1312)

allocate(rn(num_pop))

do j1=1,pointert(1)
	age_state(j1,1)=1
enddo
do age=2,max_age
	do j1=pointert(age-1)+1,pointert(age)
		age_state(j1,1)=age
	enddo
enddo

open (unit = 9141, file = 'check_age_distr1.txt', status="replace")
write (9141, '(f16.8)') (real(pointert(1),dp)/real(num_pop,dp))
do age=2,max_age
write (9141,  '(f16.8)') (real((pointert(age)-pointert(age-1)),dp)/real(num_pop,dp))
enddo
close(unit=9141)

!evolution of age state
!***********************
do tm=2,time_length  !for each subsequent period

!draw vector of "death" shocks:
!*******************************


IF (UNIXD==0) THEN

iseed=76835*tm
!set the seed:
!*************
CALL RANDOM_SEED (PUT = iseed) 

ELSE  

CALL RANDOM_SEED

ENDIF

!generate #s from uniform distr.n (double pr.):
!******************************************************
CALL RANDOM_NUMBER(rn)


!check that the right number of people die at each age, and correct value of rn accordingly:
!at age1:
check_dead=0  !counts how many people dies at each age
check_alive=0  !counts how many people are alive at each age
age=1

do j1=1,num_pop   !for each individual:
	if (age_state(j1,tm-1)==age .and. rn(j1)>p_surv(age)) then  !individual of age=age dies
		check_dead=check_dead+1
	elseif (age_state(j1,tm-1)==age .and. rn(j1)<=p_surv(age)) then
		check_alive=check_alive+1
	endif
enddo

!excess number of dead:
excess_dead=check_dead-idnint((1.0_dp-p_surv(age))*pointert(1))

if (abs(excess_dead)>0) then
allocate(integer_numbers(abs(excess_dead)),rnn(abs(excess_dead)),integer_numbers1(abs(excess_dead)),ind(abs(excess_dead)))

if (excess_dead>0) then
!we correct dead (see below)
numero=real(check_dead,dp)
else
!we correct alive
numero=real(check_alive,dp)
endif

if (unixd==0) then

iseed=435*tm
!set the seed:
!*************
CALL RANDOM_SEED (PUT = iseed)  

else
 
CALL RANDOM_SEED

endif

!generate #s from uniform distr.n (double pr.):
!******************************************************
CALL RANDOM_NUMBER(rnn)

if (abs(excess_dead)>1) then
!sort drawn numbers
call DPSORT(rnn,abs(excess_dead),ind, KFLAG, IER) 

endif


!draw "excess_dead" random individuals whose survival will be modified:
do m=1,abs(excess_dead)

integer_numbers1(m)=rnn(m)*(numero-1.0_dp)+1.0_dp

!make sure that the m number is different from the previous one
do j3=1,m-1
if (idnint(integer_numbers1(m))==idnint(integer_numbers1(j3))) then
	if (idnint(integer_numbers1(m))<idnint(numero)) then
	integer_numbers1(m)=integer_numbers1(m)+1.0_dp
	else
	!assign the first number that has not been assigned yet
	do j1=1,idnint(numero)
	attention=0
	do j2=1,j1
	if (idnint(integer_numbers1(j2))==j1) then
	attention=1
	endif
	enddo
	if (attention==0) then
	integer_numbers1(m)=real(j1,dp)
	call DPSORT(integer_numbers1(1:m),m,ind(1:m), KFLAG, IER)
	exit
	endif
	enddo
	endif
endif
enddo

enddo  !end "excess_dead" real numbers


integer_numbers=idnint(integer_numbers1)

m=1  !counter
countage=0
!there are too few dead:
if (excess_dead<0) then
do j1=1,num_pop
if (age_state(j1,tm-1)==age .and. rn(j1)<p_surv(age)) then  !he would live....
	countage=countage+1
	if (countage==integer_numbers(m)) then
	rn(j1)=p_surv(age)+0.000001_dp  !..but we kill him!
	m=m+1
	endif
endif
if (m==abs(excess_dead)+1) then    
exit
endif
enddo
!there are too many dead
elseif (excess_dead>0) then !if excess >0
do j1=1,num_pop
if (age_state(j1,tm-1)==age .and. rn(j1)>p_surv(age)) then  !he would die....
	countage=countage+1
	if (countage==integer_numbers(m)) then
	rn(j1)=p_surv(age)-0.000001_dp  !..but we let him live!
	m=m+1
	endif
endif
if (m==excess_dead+1) then
exit
endif
enddo
endif

endif

!check:
check_dead=0
do j1=1,num_pop   !for each individual:
	if (age_state(j1,tm-1)==age .and. rn(j1)>p_surv(age)) then  !individual of age=age dies
		check_dead=check_dead+1
	endif
enddo
!excess number of dead:
excess_dead=check_dead-idnint((1.0_dp-p_surv(age))*pointert(1))
if (excess_dead<-2 .or. excess_dead>2) then
print*, "mistake tm =", tm
print*, "age =", age
endif


!at other ages:
do age=2,max_age-1

if (allocated(integer_numbers)) deallocate(integer_numbers)
if (allocated(integer_numbers1)) deallocate(integer_numbers1)
if (allocated(ind)) deallocate(ind)
if (allocated(rnn)) deallocate(rnn)


check_dead=0  !counts how many people dies at each age
check_alive=0 !counts how many people lives at each age

do j1=1,num_pop   !for each individual:
	if (age_state(j1,tm-1)==age .and. rn(j1)>p_surv(age)) then  !individual of age=age dies
		check_dead=check_dead+1
	elseif (age_state(j1,tm-1)==age .and. rn(j1)<=p_surv(age)) then
		check_alive=check_alive+1
	endif
enddo

!excess number of dead:
excess_dead=check_dead-idnint((1.0_dp-p_surv(age))*(pointert(age)-pointert(age-1)))

if (excess_dead>0) then
!we correct dead (see below)
numero=real(check_dead,dp)
else
!we correct alive
numero=real(check_alive,dp)
endif

if (abs(excess_dead)>0) then
allocate(integer_numbers(abs(excess_dead)),rnn(abs(excess_dead)),integer_numbers1(abs(excess_dead)),ind(abs(excess_dead)))

!draw "excess_dead" random individuals whose survival will be modified:

if (unixd==0) then

iseed=4315*tm*age
!set the seed:
!*************
CALL RANDOM_SEED (PUT = iseed) 

else
  
CALL RANDOM_SEED

endif

!generate #s from uniform distr.n (double pr.):
!******************************************************
CALL RANDOM_NUMBER(rnn)

if (abs(excess_dead)>1) then
!sort drawn numbers
call DPSORT(rnn,abs(excess_dead),ind, KFLAG, IER) 

endif

!draw "excess_dead" random individuals whose survival will be modified:
do m=1,abs(excess_dead)

integer_numbers1(m)=rnn(m)*(numero-1.0_dp)+1.0_dp

!make sure that the m number is different from the previous one
do j3=1,m-1
if (idnint(integer_numbers1(m))==idnint(integer_numbers1(j3))) then
	if (idnint(integer_numbers1(m))<idnint(numero)) then
	integer_numbers1(m)=integer_numbers1(m)+1.0_dp
	else
	!assign the first number that has not been assigned yet
	do j1=1,idnint(numero)
	attention=0
	do j2=1,j1
	if (idnint(integer_numbers1(j2))==j1) then
	attention=1
	endif
	enddo
	if (attention==0) then
	integer_numbers1(m)=real(j1,dp)
	call DPSORT(integer_numbers1(1:m),m,ind(1:m), KFLAG, IER)
	exit
	endif
	enddo
	endif
endif
enddo

enddo  !end "excess_dead" real numbers

integer_numbers=idnint(integer_numbers1)

m=1  !counter
countage=0

!there are too few dead:
if (excess_dead<0) then
do j1=1,num_pop
if (age_state(j1,tm-1)==age .and. rn(j1)<p_surv(age)) then  !he would live....
	countage=countage+1
	if (countage==integer_numbers(m)) then
	rn(j1)=p_surv(age)+0.0001_dp  !..but we kill him!
	m=m+1
	endif
endif
if (m==abs(excess_dead)+1) then    
exit
endif
enddo
!there are too many dead
elseif (excess_dead>0) then !if excess >0
do j1=1,num_pop
if (age_state(j1,tm-1)==age .and. rn(j1)>p_surv(age)) then  !he would die....
	countage=countage+1
	if (countage==integer_numbers(m)) then
	rn(j1)=p_surv(age)-0.001_dp  !..but we let him live!
	m=m+1
	endif
endif
if (m==excess_dead+1) then
exit
endif
enddo
endif

endif

!check:
check_dead=0
do j1=1,num_pop   !for each individual:
	if (age_state(j1,tm-1)==age .and. rn(j1)>p_surv(age)) then  !individual of age=age dies
		check_dead=check_dead+1
	endif
enddo
!excess number of dead:
excess_dead=check_dead-idnint((1.0_dp-p_surv(age))*(pointert(age)-pointert(age-1)))
if (excess_dead<-2 .or. excess_dead>2) then
print*, "mistake tm =", tm
print*, "age =", age
endif


enddo   !end age loop for check

if (allocated(integer_numbers)) deallocate(integer_numbers)
if (allocated(integer_numbers1)) deallocate(integer_numbers1)
if (allocated(ind)) deallocate(ind)
if (allocated(rnn)) deallocate(rnn)


do j1=1,num_pop   !assign age to each individual at time tm

	if (rn(j1)>p_surv(age_state(j1,tm-1))) then

		age_state(j1,tm)=1  !ind is re-born		

	else

		age_state(j1,tm)=age_state(j1,tm-1)+1   !ind survives

	endif

enddo  !end individuals



enddo  !end time periods

!randomly check the age distribution at any tm:
check_age1=0
do j1=1,num_pop
	check_age1(age_state(j1,2))=check_age1(age_state(j1,2))+1
enddo		

open (unit = 9141, file = 'check_age_distr.txt', status="replace")
write (9141,  '(f16.8)') (real(check_age1,dp)/real(num_pop,dp))
close(unit=9141)

!**************************************************************************
!ASSIGN SHOCKS
!************************************************************************** 

allocate(statey(num_pop,time_length))
county=0

if (nbeta>1) then
guy2=idnint(ppbeta(1)*num_pop)+1
print*,"guy2=",guy2
else
guy2=20
endif

!cumulative probabilities calculated once and for all, for idios. shock only:

piaa=pAA
pio=py
do iy=1,nny
pia(iy,:)=pyy(iy,:)
enddo

do it=2,nny
pio(it)=pio(it-1)+py(it)
do iy=1,nny
pia(iy,it)=pia(iy,it-1)+pyy(iy,it)
enddo
enddo
do it=2,ns
do iy=1,ns
piaa(iy,it)=piaa(iy,it-1)+pAA(iy,it)
enddo
enddo


if (unixd==0) then

iseed=3461808
!set the seed:
!*************
CALL RANDOM_SEED (PUT = iseed)   

else

CALL RANDOM_SEED

endif

!generate #s from uniform distr.n (double pr.):
!******************************************************
CALL RANDOM_NUMBER(rn)


!first period: assign stationary distribution of earnings
!*********************************************************

indA=iniA  !initial aggregate shock

!************
!impatients:
!************
do j1=1,idnint(pbetastat(1)*num_pop)

it=1

do while (rn(j1)>pio(it)) 

it=it+1

enddo

statey(j1,1)=it
state(j1,1)=it+nny*(indA-1)
county(it)=county(it)+1		

enddo  !end int(pbetastat(1)*num_pop)

!check that distribution of statey is correct
do ij=nny,2,-1

!excess draws for state ij
excess(ij)=county(ij)-idnint(py(ij)*pbetastat(1)*num_pop)

if (excess(ij)>0) then
!we correct those that have state ij (see below)
numero=real(county(ij),dp)
else
!we correct those with state ij-1
numero=real(county(ij-1),dp)
endif

if (abs(excess(ij))>0) then
allocate(integer_numbers(abs(excess(ij))),rnn(abs(excess(ij))),integer_numbers1(abs(excess(ij))),ind(abs(excess(ij))))

!draw "excess_dead" random individuals whose survival will be modified:

if (unixd==0) then

iseed=43151*ij
!set the seed:
!*************
CALL RANDOM_SEED (PUT = iseed) 

else
  
CALL RANDOM_SEED

endif

!generate #s from uniform distr.n (double pr.):
!******************************************************
CALL RANDOM_NUMBER(rnn)

if (abs(excess(ij))>1) then
!sort drawn numbers
call DPSORT(rnn,abs(excess(ij)),ind, KFLAG, IER) 

endif

!draw "excess_dead" random individuals whose survival will be modified:
do m=1,abs(excess(ij))

integer_numbers1(m)=rnn(m)*(numero-1.0_dp)+1.0_dp

!make sure that the m number is different from the previous one
do j3=1,m-1
if (idnint(integer_numbers1(m))==idnint(integer_numbers1(j3))) then
	if (idnint(integer_numbers1(m))<idnint(numero)) then
	integer_numbers1(m)=integer_numbers1(m)+1.0_dp
	else
	!assign the first number that has not been assigned yet
	do j1=1,idnint(numero)
	attention=0
	do j2=1,j1
	if (idnint(integer_numbers1(j2))==j1) then
	attention=1
	endif
	enddo
	if (attention==0) then
	integer_numbers1(m)=real(j1,dp)
	call DPSORT(integer_numbers1(1:m),m,ind(1:m), KFLAG, IER)
	exit
	endif
	enddo
	endif
endif
enddo

enddo  !end "excess" real numbers

integer_numbers=idnint(integer_numbers1)

m=1  !counter
counti=0


if (excess(ij)<0) then
do j1=1,idnint(pbetastat(1)*num_pop)
if (statey(j1,1)==ij-1) then
	counti(ij)=counti(ij)+1
	if (counti(ij)==integer_numbers(m)) then
	statey(j1,1)=ij
    state(j1,1)=ij+nny*(indA-1)
	m=m+1
	endif
endif
if (m==abs(excess(ij))+1) then
county(ij-1)=county(ij-1)+excess(ij)    
exit
endif
enddo
elseif (excess(ij)>0) then !if excess >0
do j1=1,idnint(pbetastat(1)*num_pop)
if (statey(j1,1)==ij) then
	counti(ij)=counti(ij)+1
	if (counti(ij)==integer_numbers(m)) then
	statey(j1,1)=ij-1
    state(j1,1)=ij-1+nny*(indA-1)
	m=m+1
	endif
endif
if (m==excess(ij)+1) then
county(ij-1)=county(ij-1)+excess(ij)    
exit
endif
enddo
endif

endif !end correction (IF ABS(EXCESS(IJ)>0)

if (allocated(integer_numbers)) deallocate(integer_numbers)
if (allocated(integer_numbers1)) deallocate(integer_numbers1)
if (allocated(rnn)) deallocate(rnn)
if (allocated(ind)) deallocate(ind)

enddo  !end ij


!check:
county=0
do ij=1,nny
do j1=1,idnint(pbetastat(1)*num_pop)
	if ((state(j1,1)-nny*(indA-1))==ij)  then 
	county(ij)=county(ij)+1
	endif
enddo
excess(ij)=county(ij)-idnint(py(ij)*pbetastat(1)*num_pop)
if (excess(ij)<-(nny-1) .or. excess(ij)>nny-1) then
print*, "mistake"
endif
enddo

county=0
counti=0
excess=0


!************
!patients:
!************
do j1=idnint(pbetastat(1)*num_pop)+1,num_pop

it=1

do while (rn(j1)>pio(it)) 

it=it+1

enddo

statey(j1,1)=it
state(j1,1)=it+nny*ns+nny*(indA-1)
county(it)=county(it)+1				

enddo  !end patient agents

!check that distribution of statey is correct
do ij=nny,2,-1

!excess draws for state ij
excess(ij)=county(ij)-idnint(py(ij)*(1.0_dp-pbetastat(1))*num_pop)

if (excess(ij)>0) then
!we correct those that have state ij (see below)
numero=real(county(ij),dp)
else
!we correct those with state ij-1
numero=real(county(ij-1),dp)
endif

if (abs(excess(ij))>0) then
allocate(integer_numbers(abs(excess(ij))),rnn(abs(excess(ij))),integer_numbers1(abs(excess(ij))),ind(abs(excess(ij))))

!draw "excess_dead" random individuals whose survival will be modified:

if (unixd==0) then

iseed=43151*ij
!set the seed:
!*************
CALL RANDOM_SEED (PUT = iseed) 

else
  
CALL RANDOM_SEED

endif

!generate #s from uniform distr.n (double pr.):
!******************************************************
CALL RANDOM_NUMBER(rnn)

if (abs(excess(ij))>1) then
!sort drawn numbers
call DPSORT(rnn,abs(excess(ij)),ind, KFLAG, IER) 

endif

!draw "excess_dead" random individuals whose survival will be modified:
do m=1,abs(excess(ij))

integer_numbers1(m)=rnn(m)*(numero-1.0_dp)+1.0_dp

!make sure that the m number is different from the previous one
do j3=1,m-1
if (idnint(integer_numbers1(m))==idnint(integer_numbers1(j3))) then
	if (idnint(integer_numbers1(m))<idnint(numero)) then
	integer_numbers1(m)=integer_numbers1(m)+1.0_dp
	else
	!assign the first number that has not been assigned yet
	do j1=1,idnint(numero)
	attention=0
	do j2=1,j1
	if (idnint(integer_numbers1(j2))==j1) then
	attention=1
	endif
	enddo
	if (attention==0) then
	integer_numbers1(m)=real(j1,dp)
	call DPSORT(integer_numbers1(1:m),m,ind(1:m), KFLAG, IER)
	exit
	endif
	enddo
	endif
endif
enddo

enddo  !end "excess" real numbers

integer_numbers=idnint(integer_numbers1)

m=1  !counter
counti=0


if (excess(ij)<0) then
do j1=idnint(pbetastat(1)*num_pop)+1,num_pop
if (statey(j1,1)==ij-1) then
	counti(ij)=counti(ij)+1
	if (counti(ij)==integer_numbers(m)) then
	statey(j1,1)=ij
	!give state again:
    state(j1,1)=ij+nny*ns+nny*(indA-1)
	m=m+1
	endif
endif
if (m==abs(excess(ij))+1) then
county(ij-1)=county(ij-1)+excess(ij)    
exit
endif
enddo
elseif (excess(ij)>0) then !if excess >0
do j1=idnint(pbetastat(1)*num_pop)+1,num_pop
if (statey(j1,1)==ij) then
	counti(ij)=counti(ij)+1
	if (counti(ij)==integer_numbers(m)) then
	statey(j1,1)=ij-1
    !give state again:
    state(j1,1)=ij-1+nny*ns+nny*(indA-1)
	m=m+1
	endif
endif
if (m==excess(ij)+1) then
county(ij-1)=county(ij-1)+excess(ij)    
exit
endif
enddo
endif

endif !end correction (IF ABS(EXCESS(IJ)>0)

if (allocated(integer_numbers)) deallocate(integer_numbers)
if (allocated(integer_numbers1)) deallocate(integer_numbers1)
if (allocated(rnn)) deallocate(rnn)
if (allocated(ind)) deallocate(ind)

enddo  !end ij


!check:
county=0
do ij=1,nny
do j1=idnint(pbetastat(1)*num_pop)+1,num_pop
	if (state(j1,1)-nny*(indA-1)-nny*ns==ij) then
	county(ij)=county(ij)+1
	endif
enddo
excess(ij)=county(ij)-idnint(py(ij)*(1.0_dp-pbetastat(1))*num_pop)
if (excess(ij)<-(nny-1) .or. excess(ij)>nny-1) then
print*, "mistake"
endif
enddo


!draw aggregate shocks

if (unixd==0) then

iseed=985867345

CALL RANDOM_SEED (PUT = iseed)

else

CALL RANDOM_SEED

endif

CALL RANDOM_NUMBER(rA)

!subsequent periods earnings draws:
!**********************************

do tm=2,Time_length


if (allocated(integer_numbers)) deallocate(integer_numbers)
if (allocated(integer_numbers1)) deallocate(integer_numbers1)
if (allocated(rnn)) deallocate(rnn)
if (allocated(ind)) deallocate(ind)

county=0
counti=0

it=1
do while (rA(tm)>piaa(indA,it)) 

it=it+1

enddo

indA=it

if (unixd==0) then

iseed=iseed+tm

CALL RANDOM_SEED (PUT = iseed)  
else
 
CALL RANDOM_SEED
endif

CALL RANDOM_NUMBER(rn)

!****************
!impatients
!****************

do j1=1,idnint(pbetastat(1)*num_pop)

it=1

do while (rn(j1)>pia(statey(j1,tm-1),it))

it=it+1

enddo

statey(j1,tm)=it
state(j1,tm)=it+nny*(indA-1)
county(it)=county(it)+1
	
				
enddo  !end j1

!check that distribution of statey is correct
do ij=nny,2,-1
excess(ij)=county(ij)-idnint(py(ij)*pbetastat(1)*num_pop)

if (excess(ij)>0) then
!we correct those that have state ij (see below)
numero=real(county(ij),dp)
else
!we correct those with state ij-1
numero=real(county(ij-1),dp)
endif

if (abs(excess(ij))>0) then
allocate(integer_numbers(abs(excess(ij))),rnn(abs(excess(ij))),integer_numbers1(abs(excess(ij))),ind(abs(excess(ij))))

!draw "excess_dead" random individuals whose survival will be modified:
if (unixd==0) then

iseed=3151*ij*tm
!set the seed:
!*************
CALL RANDOM_SEED (PUT = iseed)  

else
 
CALL RANDOM_SEED

endif

!generate #s from uniform distr.n (double pr.):
!******************************************************
CALL RANDOM_NUMBER(rnn)

if (abs(excess(ij))>1) then
!sort drawn numbers
call DPSORT(rnn,abs(excess(ij)),ind, KFLAG, IER) 

endif

!draw "excess_dead" random individuals whose survival will be modified:
do m=1,abs(excess(ij))

integer_numbers1(m)=rnn(m)*(numero-1.0_dp)+1.0_dp

!make sure that the m number is different from the previous one
do j3=1,m-1
if (idnint(integer_numbers1(m))==idnint(integer_numbers1(j3))) then
	if (idnint(integer_numbers1(m))<idnint(numero)) then
	integer_numbers1(m)=integer_numbers1(m)+1.0_dp
	else
	!assign the first number that has not been assigned yet
	do j1=1,idnint(numero)
	attention=0
	do j2=1,j1
	if (idnint(integer_numbers1(j2))==j1) then
	attention=1
	endif
	enddo
	if (attention==0) then
	integer_numbers1(m)=real(j1,dp)
	call DPSORT(integer_numbers1(1:m),m,ind(1:m), KFLAG, IER)
	exit
	endif
	enddo
	endif
endif
enddo

enddo  !end "excess_dead" real numbers

integer_numbers=idnint(integer_numbers1)

m=1  !counter
counti=0

if (excess(ij)<0) then
do j1=1,idnint(pbetastat(1)*num_pop)
if (statey(j1,tm)==ij-1) then
    counti(ij)=counti(ij)+1
	if (counti(ij)==integer_numbers(m)) then
	statey(j1,tm)=ij
    !give state again:
    state(j1,tm)=ij+nny*(indA-1)
	m=m+1
	endif		
endif
if (m==abs(excess(ij))+1) then
county(ij-1)=county(ij-1)+excess(ij)    
exit
endif
enddo
elseif (excess(ij)>0) then  !if excess >0
do j1=1,idnint(pbetastat(1)*num_pop)
if (statey(j1,tm)==ij) then
    counti(ij)=counti(ij)+1
	if (counti(ij)==integer_numbers(m)) then
	statey(j1,tm)=ij-1
    !give state again:
    state(j1,tm)=ij-1+nny*(indA-1)
	m=m+1
	endif		
endif
if (m==excess(ij)+1) then
county(ij-1)=county(ij-1)+excess(ij)    
exit
endif
enddo
endif

endif !end correction (IF ABS(EXCESS(IJ)>0)

if (allocated(integer_numbers)) deallocate(integer_numbers)
if (allocated(integer_numbers1)) deallocate(integer_numbers1)
if (allocated(rnn)) deallocate(rnn)
if (allocated(ind)) deallocate(ind)

enddo  !END IJ

!check:
county=0
do ij=1,nny
do j1=1,idnint(pbetastat(1)*num_pop)
	if ((state(j1,tm)-nny*(indA-1))==ij) then 
	county(ij)=county(ij)+1
	endif
enddo
excess(ij)=county(ij)-idnint(py(ij)*pbetastat(1)*num_pop)
if (excess(ij)<-(nny-1) .or. excess(ij)>nny-1) then
print*, "mistake at tm,", tm
endif
enddo

county=0
counti=0
excess=0

!****************
!patients
!****************

do j1=idnint(pbetastat(1)*num_pop)+1,num_pop

it=1

do while (rn(j1)>pia(statey(j1,tm-1),it))

it=it+1

enddo

statey(j1,tm)=it
state(j1,tm)=it+nny*ns+nny*(indA-1)	
county(it)=county(it)+1	
	
				
enddo  !end j1

!check that distribution of statey is correct
do ij=nny,2,-1
excess(ij)=county(ij)-idnint(py(ij)*(1.0_dp-pbetastat(1))*num_pop)

if (excess(ij)>0) then
!we correct those that have state ij (see below)
numero=real(county(ij),dp)
else
!we correct those with state ij-1
numero=real(county(ij-1),dp)
endif

if (abs(excess(ij))>0) then
allocate(integer_numbers(abs(excess(ij))),rnn(abs(excess(ij))),integer_numbers1(abs(excess(ij))),ind(abs(excess(ij))))

!draw "excess_dead" random individuals whose survival will be modified:

if (unixd==0) then

iseed=3151*ij*tm
!set the seed:
!*************
CALL RANDOM_SEED (PUT = iseed)   

else

CALL RANDOM_SEED

endif

!generate #s from uniform distr.n (double pr.):
!******************************************************
CALL RANDOM_NUMBER(rnn)

if (abs(excess(ij))>1) then
!sort drawn numbers
call DPSORT(rnn,abs(excess(ij)),ind, KFLAG, IER) 

endif

!draw "excess_dead" random individuals whose survival will be modified:
do m=1,abs(excess(ij))

integer_numbers1(m)=rnn(m)*(numero-1.0_dp)+1.0_dp

!make sure that the m number is different from the previous one
do j3=1,m-1
if (idnint(integer_numbers1(m))==idnint(integer_numbers1(j3))) then
	if (idnint(integer_numbers1(m))<idnint(numero)) then
	integer_numbers1(m)=integer_numbers1(m)+1.0_dp
	else
	!assign the first number that has not been assigned yet
	do j1=1,idnint(numero)
	attention=0
	do j2=1,j1
	if (idnint(integer_numbers1(j2))==j1) then
	attention=1
	endif
	enddo
	if (attention==0) then
	integer_numbers1(m)=real(j1,dp)
	call DPSORT(integer_numbers1(1:m),m,ind(1:m), KFLAG, IER)
	exit
	endif
	enddo
	endif
endif
enddo

enddo  !end "excess_dead" real numbers

integer_numbers=idnint(integer_numbers1)

m=1  !counter
counti=0

if (excess(ij)<0) then
do j1=idnint(pbetastat(1)*num_pop)+1,num_pop
if (statey(j1,tm)==ij-1) then
    counti(ij)=counti(ij)+1
	if (counti(ij)==integer_numbers(m)) then
	statey(j1,tm)=ij
    !give state again:	
    state(j1,tm)=ij+nny*ns+nny*(indA-1)  
	m=m+1
	endif		
endif
if (m==abs(excess(ij))+1) then
county(ij-1)=county(ij-1)+excess(ij)    
exit
endif
enddo
elseif (excess(ij)>0) then  !if excess >0
do j1=idnint(pbetastat(1)*num_pop)+1,num_pop
if (statey(j1,tm)==ij) then
    counti(ij)=counti(ij)+1
	if (counti(ij)==integer_numbers(m)) then
	statey(j1,tm)=ij-1
    !give state again:
    state(j1,tm)=ij-1+nny*ns+nny*(indA-1)
	m=m+1
	endif		
endif
if (m==excess(ij)+1) then
county(ij-1)=county(ij-1)+excess(ij)    
exit
endif
enddo
endif

endif !end correction (IF ABS(EXCESS(IJ)>0)

if (allocated(integer_numbers)) deallocate(integer_numbers)
if (allocated(integer_numbers1)) deallocate(integer_numbers1)
if (allocated(rnn)) deallocate(rnn)
if (allocated(ind)) deallocate(ind)

enddo  !END IJ

!check:
county=0
do ij=1,nny
do j1=idnint(pbetastat(1)*num_pop)+1,num_pop
	if (state(j1,tm)-nny*(indA-1)-nny*ns==ij) then
	county(ij)=county(ij)+1
	endif
enddo
excess(ij)=county(ij)-idnint(py(ij)*(1.0_dp-pbetastat(1))*num_pop)
if (excess(ij)<-(nny-1) .or. excess(ij)>nny-1) then
print*, "mistake at tm,", tm
endif
enddo


enddo  !end time

do tm=1,time_length
if (state(guy1,1)<=nny*ns) then
state(guy1,tm)=((nny+1)/2)+(index_Ags(state(guy1,tm))-1)*nny
else
state(guy1,tm)=((nny+1)/2)+nny*ns+(index_Ags(state(guy1,tm))-1)*nny
endif
enddo

if (nbeta>1 .and. nz==1) then
do tm=1,Time_length
if (state(guy1,1)<=nny*ns) then
state(guy2,tm)=state(guy1,tm)+nny*ns   !give different edu to guy 2 but exactly same y
else
state(guy2,tm)=state(guy1,tm)-nny*ns
endif
enddo
endif

!check that in each year the aggregate state is the same:
exxe(:,:)=0.0_dp
m=0
!if (option_reg==0) then
!do tm=1,time_length
!if (tm>TAC) then
!m=m+1
!exxe(m,1)=logA(index_Ags(state(1,tm)))
!endif
!do j1=2,num_pop
!if ( index_Ags(state(j1,tm)) .ne. index_Ags(state(j1-1,tm)) ) then
!print*,"error in generation of aggregate states"
!endif
!enddo
!enddo
!else if (nhh>1) then
!do tm=1,time_length
!if (tm>TAC) then
!m=m+1
!exxe(m,index_Ags(state(1,tm)))=1.0_dp
!endif
!do j1=2,num_pop
!if ( index_Ags(state(j1,tm)) .ne. index_Ags(state(j1-1,tm)) ) then
!print*,"error in generation of aggregate states"
!endif
!enddo
!enddo
!else
do tm=1,time_length
if (tm>TAC) then
m=m+1
exxe(m,index_Ags(state(1,tm)))=1.0_dp
endif
do j1=2,num_pop
if ( index_Ags(state(j1,tm)) .ne. index_Ags(state(j1-1,tm)) ) then
print*,"error in generation of aggregate states"
endif
enddo
enddo
!endif

!MARINA
!THANKSGIVING
!if (option_reg==1) then
do is=1,ns
 exxe(1:(time_length-TAC-1),is)= exxe(2:(time_length-TAC),is)
enddo
!endif

!THANKSGIVING
!if (option_reg==0) then
!exxe(1:(time_length-TAC-1),1)= exxe(2:(time_length-TAC),1)
!endif


!Marina June 2009
!check that the percentage of betas is correct and compute counters for simul distribution computations:

allocate(indbetalow(num_pop,time_length),indbetahigh(num_pop,time_length))


indbetalow=0
indbetahigh=1

count_a_people=0
count_ab_people=0
count_b_people=0
count_s=0

!at time tm =1:
do j1=1,num_pop
count_b_people(index_beta(state(j1,1)))=count_b_people(index_beta(state(j1,1)))+1  !count of betas at any tm (not varying)
if (index_beta(state(j1,1))==1) then
m=m+1
indbetalow(j1,1)=1
indbetahigh(j1,1)=0
endif
enddo


do tm=2,time_length
if (tm>=TAC) then
count_s(index_Ags(state(1,tm)))=count_s(index_Ags(state(1,tm)))+1   !counter of number of aggregate shocks 
endif
m=0
do j1=1,num_pop
if (tm>=TAC) then
count_a_people(age_state(j1,tm),tm)=count_a_people(age_state(j1,tm),tm)+1    !counter of people for each age and time tm
count_ab_people(age_state(j1,tm),index_beta(state(j1,tm)),tm)=count_ab_people(age_state(j1,tm),index_beta(state(j1,tm)),tm)+1    !counter of people for each age and beta at time tm
endif
if (index_beta(state(j1,tm))==1) then
m=m+1
indbetalow(j1,tm)=1
indbetahigh(j1,tm)=0
endif
enddo
!if (m<int(pbetastat(1)*num_pop) .or. m>int(pbetastat(1)*num_pop)) then
!print*, "error in generation of betas at t=",tm
!endif
enddo

!Marina June 2009
open (unit = 13129, file = 'count_a_people.txt', status = 'replace')
do tm =TAC, time_length
write (13129,  '(I5)') (count_a_people(:,tm))
enddo
close (unit=13129)
!Marina June 2009
open (unit = 13129, file = 'count_ab_people.txt', status = 'replace')
do tm =TAC, time_length
do ib=1,nbeta
write (13129,  '(I5)') (count_ab_people(:,ib,tm))
enddo
enddo
close (unit=13129)
!Marina June 2009
open (unit = 131291, file = 'count_s.txt', status = 'replace')
write (131291,  '(I5)') (count_s)
close (unit=131291)
!Marina June 2009
open (unit = 131291, file = 'count_b_people.txt', status = 'replace')
write (131291,  '(I5)') (count_b_people)
close (unit=131291)

open (unit = 9141, file = 'sim_a.txt', status="replace")
do tm=1,time_length
write (9141,  '(f16.4)') (Ags(index_Ags(state(2,tm))))
enddo
close (unit=9141)

open (unit = 9141, file = 'sim_z.txt', status="replace")
do tm=time_start+1,time_length
do j1=idnint((num_pop-num_pop1)*pbetastat(1)),idnint((num_pop-num_pop1)*pbetastat(1))+num_pop1-1
write (9141,  '(f16.4)') (zy(index_Azy(state(j1,tm))))
enddo
enddo
close(unit=9141)

open (unit = 9141, file = 'sim_z_1.txt', status="replace")
do tm=1,time_length
write (9141,  '(f16.4)') (zy(index_Azy(state(guy1,tm))))
enddo
close(unit=9141)

open (unit = 9141, file = 'sim_z_2.txt', status="replace")
do tm=1,time_length
write (9141,  '(f16.4)') (zy(index_Azy(state(guy2,tm))))
enddo
close(unit=9141)

open (unit = 9141, file = 'sim_beta_1.txt', status="replace")
do tm=1,time_length
write (9141,  '(f16.4)') (beta(index_beta(state(guy1,tm))))
enddo
close(unit=9141)

open (unit = 9141, file = 'sim_beta_2.txt', status="replace")
do tm=1,time_length
write (9141,  '(f16.4)') (beta(index_beta(state(guy2,tm))))
enddo
close(unit=9141)

open (unit = 9141, file = 'sim_Ags_1.txt', status="replace")
do tm=1,time_length
write (9141,  '(f16.4)') (Ags(index_Ags(state(guy1,tm))))
enddo
close(unit=9141)

open (unit = 9141, file = 'sim_Ags_2.txt', status="replace")
do tm=1,time_length
write (9141,  '(f16.4)') (Ags(index_Ags(state(guy2,tm))))
enddo
close(unit=9141)


if (allocated(rn)) deallocate(rn)
if (allocated(statey)) deallocate(statey)


END SUBROUTINE random_gen1
!*****************************************************************
